//+------------------------------------------------------------------+
//|                                                MACD_AvMaxMin.mq4 |
//|                                                           Kirill |
//|                                          StockProgrammer@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Kirill"
#property link      "StockProgrammer@mail.ru"
//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 4
#property  indicator_color1  Silver
#property  indicator_color2  Red
#property  indicator_color3  Blue
#property  indicator_color4  Purple
#property  indicator_width1  2
//---- indicator parameters
extern int FastEMA=12;
extern int SlowEMA=26;
extern int SignalSMA=9;
extern int AvBars=100;  //  . 
//---- indicator buffers
double     MacdBuffer[];
double     SignalBuffer[];
double     AvMax[];
double     AvMin[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- drawing settings
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexStyle(2,DRAW_LINE, EMPTY, 4);
   SetIndexStyle(3,DRAW_LINE, EMPTY, 4);      
   SetIndexDrawBegin(1,SignalSMA);
   IndicatorDigits(Digits+1);
//---- indicator buffers mapping
   SetIndexBuffer(0,MacdBuffer);
   SetIndexBuffer(1,SignalBuffer);
   SetIndexBuffer(2,AvMax);
   SetIndexBuffer(3,AvMin);   
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD_AvMaxMin("+FastEMA+","+SlowEMA+","+SignalSMA+","+AvBars+")");
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");
   SetIndexLabel(2,"AvMax");
   SetIndexLabel(3,"AvMin");   
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   int counted_bars=IndicatorCounted();
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- macd counted in the 1-st buffer
   for(int i=0; i<limit; i++)
      MacdBuffer[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
//---- signal line counted in the 2-nd buffer
   for(i=0; i<limit; i++)
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalSMA,0,MODE_SMA,i);
//---- AvMax & AvMin
   for(i=0; i<limit; i++)
   {      
      AvMax[i] = countAvMax(i);
      AvMin[i] = countAvMin(i);
   }     
//---- done
   return(0);
  }
//+------------------------------------------------------------------+
double countAvMax(int j)
{
   int i;
   double cnt = 0.0, m = 0.0;
   
   for(i=j+2; i<=j+AvBars; i++)
   {
      if(SignalBuffer[i]>SignalBuffer[i+1] && SignalBuffer[i]>SignalBuffer[i-1])    //SignalBuffer[i] -  . MAX
      {
         m+=SignalBuffer[i];
         cnt++;
      }
   }      
   if(cnt>0.0) return(NormalizeDouble(m/cnt,Digits));
   else return(0.0);
}

double countAvMin(int j)
{
   int i;
   double cnt = 0.0, m = 0.0;
   
   for(i=j+2; i<=j+AvBars; i++)
   {
      if(SignalBuffer[i]<SignalBuffer[i+1] && SignalBuffer[i]<SignalBuffer[i-1])    //SignalBuffer[i] -  . MIN
      {
         m+=SignalBuffer[i];
         cnt++;
      }
   }      
   if(cnt>0.0) return(NormalizeDouble(m/cnt,Digits));
   else return(0.0);
}