//+------------------------------------------------------------------+
//|                                                      MACD_Lnx.mq4|
//|                                                                  |
//|                                                                  |
//|                                                mladen & Linuxser |
//|                                                                  |
//|                                                         May 2008 |
//+------------------------------------------------------------------+
#property copyright "Under The GNU General Public License"
#property link      "www.gnu.org"

//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 8
#property  indicator_color1  Red
#property  indicator_color2  Green
#property  indicator_color3  Red
#property  indicator_color4  Magenta
#property  indicator_color5  Green
#property  indicator_color6  MediumOrchid
#property  indicator_color7  Blue
#property  indicator_color8  Red
#property  indicator_width1  1
#property  indicator_width2  1
#property  indicator_width3  1
#property  indicator_width4  1
#property  indicator_width5  1
#property  indicator_width6  1
#property  indicator_width7  1
#property  indicator_width8  1
#property  indicator_level1  0
#property  indicator_levelcolor  Silver
#property  indicator_levelstyle 2

//---- indicator parameters
extern int FastMA=12;
extern int SlowMA=26;
extern int SignalEMA=9;
extern int Applied_Price  =  0; // 0: Close // 1: Open // 2: High // 3: Low // 4: Median (H+L/2) // 5: Typical (H+L+C/3) // 6: Weighted (H+L+C+C/4)
extern int MA_Method  =  1; //MODE_SMA :	0 // MODE_EMA:	1 // MODE_SMMA: 2 // MODE_LWMA: 3 //
extern int Style  = 0; 

//---- indicator buffers
double     MACDBuffer[];
double     SignalBuffer[];
double     StyleBuffer[];
double     OsmaBuffer[];
double     ExtBuffer1[];
double     ExtBuffer2[];
double     ExtBuffer3[];
double     ExtBuffer4[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorDigits(Digits+2);
      SetIndexBuffer(0,StyleBuffer);
      SetIndexBuffer(1,ExtBuffer1);
      SetIndexBuffer(2,ExtBuffer2);  
      SetIndexBuffer(3,ExtBuffer3);
      SetIndexBuffer(4,ExtBuffer4);
      SetIndexBuffer(5,OsmaBuffer);
      SetIndexBuffer(6,MACDBuffer);
      SetIndexBuffer(7,SignalBuffer);
            setUpStyles();
   IndicatorShortName("MACD Lnx ("+FastMA+","+SlowMA+","+SignalEMA+") Method ["+MA_Method+"])");
   return(0);
}

//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   double alpha = 2.0 /(1.0 + SignalEMA);
   int    counted_bars = IndicatorCounted();
   int    i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=Bars-counted_bars;

   //
   //
   //
   //
   //
      
   for(i=limit; i>=0; i--)
   {
      MACDBuffer[i]   = iMA(NULL,0,FastMA,0,MA_Method,Applied_Price,i)-iMA(NULL,0,SlowMA,0,MA_Method,Applied_Price,i);
      SignalBuffer[i] = SignalBuffer[i+1]+alpha*(MACDBuffer[i]-SignalBuffer[i+1]);
      OsmaBuffer[i]   = MACDBuffer[i]-SignalBuffer[i];
      StyleBuffer[i]  = MACDBuffer[i];
      
         //
         //
         //
         //
         //
         
            ExtBuffer1[i]=EMPTY_VALUE;
            ExtBuffer2[i]=EMPTY_VALUE;
            ExtBuffer3[i]=EMPTY_VALUE;
            ExtBuffer4[i]=EMPTY_VALUE;
            
            switch (Style)            
            {
               case 1 :
                  if (OsmaBuffer[i]>0)
                        ExtBuffer3[i]=OsmaBuffer[i];
                  else  ExtBuffer4[i]=OsmaBuffer[i];
                  break;
               case 4 :
                  if (OsmaBuffer[i]<OsmaBuffer[i+1])
                        ExtBuffer3[i]=OsmaBuffer[i];
                  else  ExtBuffer4[i]=OsmaBuffer[i];
                  break;
               case 5 :
               case 6 :
                  if (MACDBuffer[i]>MACDBuffer[i+1])
                        ExtBuffer1[i]=MACDBuffer[i];
                  else  ExtBuffer2[i]=MACDBuffer[i];
                  break;
            }                  
   }                                     
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void setUpStyles()
{
   for (int i=0; i<8;i++) { SetIndexStyle(i,DRAW_NONE); SetIndexLabel(i,NULL); }         

   //
   //
   //
   //
   //
 
   Style = MathMax(MathMin(Style,8),0);
   switch (Style)
   {
      case 0:  SetIndexStyle(5,DRAW_HISTOGRAM); SetIndexLabel(5,"OSMA");
               SetIndexStyle(6,DRAW_LINE);      SetIndexLabel(6,"MACD");
               SetIndexStyle(7,DRAW_LINE);      SetIndexLabel(7,"Signal");
               break;      
      case 1:  SetIndexStyle(3,DRAW_HISTOGRAM); SetIndexLabel(3,"MACD Up");
               SetIndexStyle(4,DRAW_HISTOGRAM); SetIndexLabel(4,"MACD Down");
               SetIndexStyle(6,DRAW_LINE);      SetIndexLabel(6,"MACD");
               SetIndexStyle(7,DRAW_LINE);      SetIndexLabel(7,"Signal");
               break;      
      case 2:  SetIndexStyle(6,DRAW_LINE);      SetIndexLabel(6,"MACD");
               SetIndexStyle(7,DRAW_LINE);      SetIndexLabel(7,"Signal");
               break;      
      case 3:  SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,"MACD");
               SetIndexStyle(5,DRAW_HISTOGRAM); SetIndexLabel(5,"OSMA");
               SetIndexStyle(6,DRAW_LINE);      SetIndexLabel(6,"MACD");
               SetIndexStyle(7,DRAW_LINE);      SetIndexLabel(7,"Signal");
               break;
      case 4:  SetIndexStyle(3,DRAW_HISTOGRAM); SetIndexLabel(3,"MACD Up");
               SetIndexStyle(4,DRAW_HISTOGRAM); SetIndexLabel(4,"MACD Down");
               SetIndexStyle(6,DRAW_LINE);      SetIndexLabel(6,"MACD");
               SetIndexStyle(7,DRAW_LINE);      SetIndexLabel(7,"Signal");
               break;      
      case 5:  SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"MACD Up");
               SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexLabel(2,"MACD Down");
               break;
      case 6:  SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"MACD Up");
               SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexLabel(2,"MACD Down");
               SetIndexStyle(7,DRAW_LINE);      SetIndexLabel(7,"Signal");
               break;
      case 7:  SetIndexStyle(0,DRAW_LINE);      SetIndexLabel(0,"MACD");
               SetIndexStyle(6,DRAW_HISTOGRAM); SetIndexLabel(6,"MACD");
               break;
      case 8:  SetIndexStyle(5,DRAW_HISTOGRAM); SetIndexLabel(5,"OSMA");
               break;
   }               
}