//+------------------------------------------------------------------+
//|                                      Macd_Correct_Hull_Style.mq4 |
//+------------------------------------------------------------------+

#property copyright "Copyright  2005, David W. Thomas"
#property link      "mailto:davidwt@usa.net"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_color3 Green

//---- input parameters

extern int IndexMultiplier = 0;

extern int FastMAPeriod   = 12;
extern int SlowMAPeriod   = 26;
extern int SignalPeriod   =  9;

extern int MA_Shift       =  0;
extern int MA_Method      =  1; // 0: sma // 1: ema // 2: smma // 3: lwma
extern int Applied_Price  =  0; // 0: Close // 1: Open // 2: High // 3: Low // 4: Median (H+L/2) // 5: Typical (H+L+C/3) // 6: Weighted (H+L+C+C/4)

extern int MacdWidth      =  1;
extern int SignalWidth    =  1;
extern int HistogramWidth =  3;

//---- buffers

double MACDLineBuffer[];
double SignalLineBuffer[];
double HistogramBuffer[];

//---- variables

double alpha = 0;
double alpha_1 = 0;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
{
   
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);
   
   //---- indicators
   
   SetIndexStyle(2,DRAW_LINE,STYLE_SOLID,MacdWidth);
   SetIndexBuffer(2,MACDLineBuffer);
   SetIndexDrawBegin(2,SlowMAPeriod);
   
   SetIndexStyle(1,DRAW_LINE,STYLE_DOT,SignalWidth);
   SetIndexBuffer(1,SignalLineBuffer);
   SetIndexDrawBegin(1,SlowMAPeriod+SignalPeriod);
   
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,HistogramWidth);
   SetIndexBuffer(0,HistogramBuffer);
   SetIndexDrawBegin(0,SlowMAPeriod+SignalPeriod);
   
   //---- name for DataWindow and indicator subwindow label
   
   IndicatorShortName("Macd_Correct_Hull_Style("+FastMAPeriod+","+SlowMAPeriod+","+SignalPeriod+")");
   
   SetIndexLabel(2,"MACD");
   SetIndexLabel(1,"Signal");
   SetIndexLabel(0,"Histogram");
   
   //----
	
	alpha = 2.0 / (SignalPeriod + 1.0);
	alpha_1 = 1.0 - alpha;
   
   //----
   
   return(0);
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+

int deinit()
{
   //---- 
   
   //----
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
{
   
   if (IndexMultiplier == 0) IndexMultiplier = 1;
   
   int limit;
   int counted_bars = IndicatorCounted();
   
   //---- check for possible errors
   
   if (counted_bars<0) return(-1);
   
   //---- last counted bar will be recounted
   
   if (counted_bars>0) counted_bars--;
   limit = Bars - counted_bars;
   
   for(int i=limit; i>=0; i--)
   {
      
      MACDLineBuffer[i]   = ((iMA(Symbol(),0,MathFloor(FastMAPeriod/2),MA_Shift,MA_Method,Applied_Price,i)*2 - iMA(Symbol(),0,FastMAPeriod,MA_Shift,MA_Method,Applied_Price,i))
                          - (iMA(Symbol(),0,MathFloor(SlowMAPeriod/2),MA_Shift,MA_Method,Applied_Price,i)*2 - iMA(Symbol(),0,SlowMAPeriod,MA_Shift,MA_Method,Applied_Price,i)))*IndexMultiplier;
      
      SignalLineBuffer[i] = alpha*MACDLineBuffer[i] + alpha_1*SignalLineBuffer[i+1];
      
      HistogramBuffer[i]  = MACDLineBuffer[i] - SignalLineBuffer[i];
      
   }
   
   //----
   
   return(0);
}

//+------------------------------------------------------------------+