//+-----------------------------------------------------------------+
//| Spaghetti MACD, 2014 Version                                    |
//| Based on Spaghetti Trading System MACD                          |
//+-----------------------------------------------------------------+
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  SlateGray
#property indicator_color2  DarkGray
#property indicator_color3  Blue
#property indicator_color4  Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_style3  STYLE_DOT
#property indicator_style4  STYLE_DOT
#property indicator_level1  0
#property indicator_levelcolor  DimGray
//---- Input Parameters
extern string TimeFrame = "Current";
extern int    FastMin   = 3 ;
extern int    FastMax   = 15;
extern int    SlowMin   = 30;
extern int    SlowMax   = 60;
extern int    Method    = 1 ;
extern int    Price     = 0 ;
//---- Buffers
double Slow     [];
double Fast     [];
double Spread   [];
double Amplitude[];
//---- Time Frame Variables
string Name  ;
bool   RBars ;
int    TFrame;
	 	   	 		  					  		 				 			  	 	   		  	 	 		   	  					  			  	 			 	 		 			  		       							 		  	 	   			    	 			 						  	 			  		  		  	   							   	 	 		 		 				   		   			      		   					 		  	 	       			  		 	    			 	  		  	 				 	   		 	 	   	  		 	  	  	       		 	  	 		 
//+-------------------------------------------------------+
//| Custom Indicator Initialization Function              |
//+-------------------------------------------------------+
int init()
  {
   SetIndexBuffer(0, Amplitude);SetIndexStyle(0,DRAW_HISTOGRAM);SetIndexLabel(0,"Amplitude");
   SetIndexBuffer(1, Spread   );SetIndexStyle(1,DRAW_HISTOGRAM);SetIndexLabel(1,"Spread"   );
   SetIndexBuffer(2, Slow     );SetIndexLabel(2,"Slow");
   SetIndexBuffer(3, Fast     );SetIndexLabel(3,"Fast");
   
   Name  = WindowExpertName();
   RBars = TimeFrame == "RBars";
   if (RBars)  return(0);
   TFrame = STTFrame(TimeFrame);
   // Indicator Name
   IndicatorShortName(TFTString(TFrame)+" Spaghetti MACD, Values:");
   return (0);
  }

//+-------------------------------------------------------+
//| Custom Indicator Deinitialization Function            |
//+-------------------------------------------------------+
int deinit()
  {
   return (0);
  }

//+-------------------------------------------------------+
//| Spaghetti MACD                                        |
//+-------------------------------------------------------+	 	 	    			  	 		 		 	  	 	 	  	 	 	 			 			 				  	 	   	   			 		    	     			   		 			 	  	  	 					 		   							 	 	  	  			 		   	  	    	
int start()
  {
   int CountedBars = IndicatorCounted();
   if (CountedBars < 0)  return(-1);
   if (CountedBars > 0)  CountedBars--;
   int Limit = MathMin(Bars-CountedBars,Bars-1);
//---- Multiple Time Frame Adjustment
   if (RBars)
     {
      Amplitude[0] = MathMin(Limit+1,Bars-1);
      return(0);
     }
   if (TFrame != Period())
     {
      Limit = MathMax(Limit,MathMin(Bars-1,iCustom(Symbol(),TFrame,Name,"RBars",0,0)*TFrame/Period()));
      for (int i=Limit; i>= 0; i--)
        {
         int   y  = iBarShift(Symbol(),TFrame,Time[i]);
         Amplitude[i] = iCustom(Symbol(),TFrame,Name,"",FastMin,FastMax,SlowMin,SlowMax,Method,Price,0,y);
         Spread   [i] = iCustom(Symbol(),TFrame,Name,"",FastMin,FastMax,SlowMin,SlowMax,Method,Price,1,y);
         Slow     [i] = iCustom(Symbol(),TFrame,Name,"",FastMin,FastMax,SlowMin,SlowMax,Method,Price,2,y);
         Fast     [i] = iCustom(Symbol(),TFrame,Name,"",FastMin,FastMax,SlowMin,SlowMax,Method,Price,3,y);
        }
//--- Multiple time frame processing is done.
      return(0);
     }
//---- Spaghetti MACD Iteration Function
   for(i=Limit; i>=0; i--)
     {
      double SMax = iMA(Symbol(),Period(),SlowMax,0,Method,Price,i);
      double SMin = iMA(Symbol(),Period(),SlowMin,0,Method,Price,i);
      double FMax = iMA(Symbol(),Period(),FastMax,0,Method,Price,i);
      double FMin = iMA(Symbol(),Period(),FastMin,0,Method,Price,i);
      Slow[i] = SMin-SMax;
      Fast[i] = FMin-FMax;
      Amplitude[i] = FMin-SMax;
      Spread   [i] = FMax-SMin;
     }
   return (0);
  }

//+-------------------------------------------------------+
//| User-Defined Functions                                |
//+-------------------------------------------------------+
//---- Timer
string TFSTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    ITFTable[] = {1,5,15,30,60,240,1440,10080,43200};
int STTFrame(string TFS)
  {
   TFS = StringUpperCase(TFS);
   for (int i=ArraySize(ITFTable)-1; i>=0; i--)
   if (TFS == TFSTable[i] || TFS == ""+ITFTable[i])  return(MathMax(ITFTable[i],Period()));
   return(Period());
  }

string TFTString(int TF)
  {
   for (int i=ArraySize(ITFTable)-1; i>=0; i--)
   if (TF == ITFTable[i])  return(TFSTable[i]);
   return("");
  }

string StringUpperCase(string STR)
  {
   string s = STR;
   for (int length=StringLen(STR)-1; length>=0; length--)
     {
      int tchar = StringGetChar(s,length);
      if ((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))  s = StringSetChar(s,length,tchar-32);
      else if (tchar > -33 && tchar < 0)  s = StringSetChar(s,length,tchar+224);
     }
   return(s);
  }
//+-------------------------------------------------------+