//+------------------------------------------------------------------+
//|                                                   Stochastic RSI |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  PaleVioletRed
#property indicator_color2  PaleVioletRed
#property indicator_width1  2
#property indicator_style2  STYLE_DOT
#property indicator_level2  20
#property indicator_level1  80

//
//
//
//
//

extern int FastLen                   = 5;
extern int SlowLen                   = 34;
extern int Length                    = 1;
extern int StochasticKPeriod         = 14;
extern int StochasticSlowing         = 3;
extern int StochasticSlowingMaMethod = 6;
extern int StochasticDPeriod         = 3;
extern int StochasticDMaMethod       = 6;

//
//
//
//
//

double stochMacd[];
double stochMacdSignal[];
double Macdbuf[];
double stoch[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,stochMacd);
   SetIndexBuffer(1,stochMacdSignal);
   SetIndexBuffer(2,Macdbuf);
   SetIndexBuffer(3,stoch);

      //
      //
      //
      //
      //
      
      string smooth1 = getAverageName(StochasticSlowingMaMethod);
      string smooth2 = getAverageName(StochasticDMaMethod);
      
      //
      //
      //
      //
      //
      
   IndicatorShortName("Stochastic zerolag Macd ("+FastLen+","+SlowLen+","+StochasticKPeriod+","+StochasticSlowing+","+smooth1+","+StochasticDPeriod+","+smooth2+")");
   return(0);
}
int deinit() { return(0);}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   for(i=limit; i>=0; i--)
   {
           double macd       = 0;
           double signal     = 0; CalculateMacd(i,macd,signal);
           Macdbuf[i]        = macd; 
           double min        = Macdbuf[ArrayMinimum(Macdbuf,StochasticKPeriod,i)];
           double max        = Macdbuf[ArrayMaximum(Macdbuf,StochasticKPeriod,i)];
           stoch[i]          = 0; if (min!=max) stoch[i] = 100.0*(Macdbuf[i]-min)/(max-min);
           stochMacd[i]      = iCMAOnArray(stoch    ,stochMacd      ,StochasticSlowing,StochasticSlowingMaMethod,i);
           stochMacdSignal[i]= iCMAOnArray(stochMacd,stochMacdSignal,StochasticDPeriod,StochasticDMaMethod      ,i);
   }
   return(0);
}




//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iCMAOnArray(double& source[], double& result[], int period, int method, int i)
{
   switch(method)
   {
      case 0: return(iSmaOnArray(source,period,i));
      case 1: return(iEmaOnArray(source,result,period,i));
      case 2: return(iSmmaOnArray(source,result,period,i));
      case 3: return(iLwmaOnArray(source,period,i));
      case 4: return(iTmaOnArray(source,period,i));
      case 5: return(iSineWMAOnArray(source,period,i));
      case 6: return(iVolumeWMAOnArray(source,period,i));
      case 7: return(iEPMAOnArray(source,period,i));
   }
   return(EMPTY_VALUE);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iSmaOnArray(double& array[], double period, int i)
{
   double sum = 0; 
   for(int k=0; k<period && (i+k)<Bars; k++) sum += array[i+k];  
   if (k!=0)
         return(sum/k);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iEmaOnArray(double& source[], double& result[], double period, int i)
{
   double alpha = 2.0 / (1.0+period);
          result[i] = result[i+1]+alpha*(source[i]-result[i+1]);
   return(result[i]);
}
//
//
//
//
//

double iSmmaOnArray(double& source[], double& result[], double period, int i)
{
   if (i>=(Bars-period))
   {
      double sum = 0; 
         for(int k=0; k<period && (i+k)<Bars; k++) sum += source[i+k];  
         if (k!=0)
               result[i] = sum/k;
         else  result[i] = EMPTY_VALUE;
   }      
   else   result[i] = (result[i+1]*(period-1)+source[i])/period;
   return(result[i]);
}

//
//
//
//
//

double iLwmaOnArray(double& array[], double period, int i)
{
   double sum  = 0;
   double sumw = 0;

   for(int k=0; k<period && (i+k)<Bars; k++)
   {
      double weight = period-k;
             sumw  += weight;
             sum   += weight*array[i+k];  
   }             
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iTmaOnArray(double& array[], double period, int i)
{
   double half = (period+1.0)/2.0;
   double sum  = 0;
   double sumw = 0;

   for(int k=0; k<period && (i+k)<Bars; k++)
   {
      double weight = k+1; if (weight > half) weight = period-k;
             sumw  += weight;
             sum   += weight*array[i+k];  
   }             
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

#define Pi 3.14159265358979323846
double iSineWMAOnArray(double& array[], int period, int i)
{
   double sum  = 0;
   double sumw = 0;
  
   for(int k=0; k<period && (i+k)<Bars; k++)
   { 
      double weight = MathSin(Pi*(k+1)/(period+1));
             sumw  += weight;
             sum   += weight*array[i+k]; 
   }
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iVolumeWMAOnArray(double& array[], int period, int i)
{
   double sum  = 0;
   double sumw = 0;
  
   for(int k=0; k<period && (i+k)<Bars; k++)
   { 
      double weight = Volume[i+k];
             sumw  += weight;
             sum   += weight*array[i+k]; 
   }
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iEPMAOnArray(double& array[], int period, int i)
{
   double weight = 2.0*period-1.0;
   double sum    = 0;
   double sumw   = 0;
  
   for(int k=0; k<period && (i+k)<Bars; k++)
   { 
       sumw   += weight;
       sum    += weight*array[i+k]; 
       weight -= 3;
   }
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}




//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string methodNames[] = {"SMA","EMA","SMMA","LWMA","TriMA","SWMA","VWMA","EPMA"};
string getAverageName(int& method)
{
   method=MathMax(MathMin(method,7),0); return(methodNames[method]);
}

  
double workBuffer[][8];

#define ema11 0
#define ema12 1
#define ema21 2
#define ema22 3
#define ema31 4
#define ema32 5
#define imacd 6
#define isign 7

  
  
void CalculateMacd(int i, double& macd, double& signal)
{
   double alpha1 = 2.0/(1.0+FastLen);
   double alpha2 = 2.0/(1.0+SlowLen);
   double alpha3 = 2.0/(1.0+Length);
   int    r=Bars-i-1;

      if (ArrayRange(workBuffer,0)!=Bars) ArrayResize(workBuffer,Bars);
      
   //
   //
   //
   //
   //
         
   
   double price = iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i);
   if (i==(Bars-1))
   {
      workBuffer[r][ema11] = price;
      workBuffer[r][ema12] = price;
      workBuffer[r][ema21] = price;
      workBuffer[r][ema22] = price;
      workBuffer[r][imacd] = 0;
      workBuffer[r][isign] = 0;
   }
   else
   {
      workBuffer[r][ema11] = workBuffer[r-1][ema11]+alpha1*(price               -workBuffer[r-1][ema11]);
      workBuffer[r][ema12] = workBuffer[r-1][ema12]+alpha1*(workBuffer[r][ema11]-workBuffer[r-1][ema12]);
      workBuffer[r][ema21] = workBuffer[r-1][ema21]+alpha2*(price               -workBuffer[r-1][ema21]);
      workBuffer[r][ema22] = workBuffer[r-1][ema22]+alpha2*(workBuffer[r][ema21]-workBuffer[r-1][ema22]);
      workBuffer[r][imacd] = (2.0*workBuffer[r][ema11]-workBuffer[r][ema12])-
                             (2.0*workBuffer[r][ema21]-workBuffer[r][ema22]);
 
      //
      //
      //
      //
      //
      
      workBuffer[r][ema31] = workBuffer[r-1][ema31]+alpha3*(workBuffer[r][imacd]-workBuffer[r-1][ema31]);
      workBuffer[r][ema32] = workBuffer[r-1][ema32]+alpha3*(workBuffer[r][ema31]-workBuffer[r-1][ema32]);
      workBuffer[r][isign] = (2.0*workBuffer[r][ema31]-workBuffer[r][ema32]);
   }
   macd   = workBuffer[r][imacd];
   signal = workBuffer[r][isign];
}

//+-------------