//+------------------------------------------------------------------+
//|                                                   SimpleMACD.mq4 |
//|                                                       Lizhniyk E |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Lizhniyk E"
#property link      "http://www.metaquotes.net"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_color3 Green
//---- input parameters
extern int       macd_period1=10;//  1
extern int       macd_period2=20;//  2
extern int       app_pr=5;//  
extern string    extrapolation="*************";
extern int       lb  =30;   //   
extern int       np =100;   // 
extern int       npstop=100;
extern int       npstep=5;
extern int       nf  =100;   //Number of bars to predict; for LP is set at PastBars-Order-1 
extern int       HarmNo   =20;    //Number of frequencies for Mathod 1; HarmNo=0 computes PastBars harmonics
extern double    FreqTOL  =0.0001;//Tolerance of frequency calculation for Method 1

int originalNp;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
double Ext[]/*, f[], pr[]*/;
double pv[], fv[];
int init()
  {
//---- indicators
   SetIndexBuffer(0,Ext);
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,pv);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(2,fv);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexShift(1,-lb);
   SetIndexShift(2,nf-lb);
   originalNp=np;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = Bars-counted_bars;

   for(i=limit;i>=0;i--)
   {
      double ma1=iMA(NULL,0,macd_period1,0,0,app_pr,i);
      double ma2=iMA(NULL,0,macd_period2,0,0,app_pr,i);
      Ext[i]=(ma1-ma2)/Point;
   }
//*************************
   for(np=originalNp;np<=npstop;np=np+npstep)
   {
      double av=0.0;
      for(i=0;i<np;i++) av+=Ext[i+lb]; av/=np;
      //******
      for(i=0;i<np;i++)
      {
         pv[i]=av;
         if(i<=nf) fv[i]=av;
      }
      //*****
      double w,m,c,s;
      for(int harm=0;harm<HarmNo;harm++)
      {
         Freq(w,m,c,s);
         for(i=0;i<np;i++) 
         {
            pv[i]+=m+c*MathCos(w*i)+s*MathSin(w*i);
            if(i<=nf) fv[i]+=m+c*MathCos(w*i)-s*MathSin(w*i);
         }         
      }  
      //******
      for(i=0;i<=(nf-1)/2;i++)
      {
         double tmp=fv[i];
            fv[i]=fv[nf-i];
            fv[nf-i]=tmp;
      } 
      //*****
   }
//----
   SetIndexDrawBegin(1,Bars-originalNp-lb-1);
   SetIndexDrawBegin(2,Bars-originalNp-1);
   return(0);
  }
//+------------------------------------------------------------------+
//Quinn and Fernandes algorithm
void Freq(double& w, double& m, double& c, double& s)
{
   double z[],num,den;
   ArrayResize(z,np);
   double a=0.0;
   double b=2.0;
   z[0]=Ext[lb]-pv[0];
   while(MathAbs(a-b)>FreqTOL)
   {
      a=b;
      z[1]=Ext[1+lb]-pv[1]+a*z[0];
      num=z[0]*z[1];
      den=z[0]*z[0];
      for(int i=2;i<np;i++)
      {
         z[i]=Ext[i+lb]-pv[i]+a*z[i-1]-z[i-2];
         num+=z[i-1]*(z[i]+z[i-2]);
         den+=z[i-1]*z[i-1];
      }
      b=num/den;
   }
   w=MathArccos(MathAbs(b/2.0));
   Fit(w,m,c,s);
   return;
}
//+-------------------------------------------------------------------------+
void Fit(double w, double& m, double& c, double& s)
{
   double Sc=0.0;
   double Ss=0.0;
   double Scc=0.0;
   double Sss=0.0;
   double Scs=0.0;
   double Sx=0.0;
   double Sxx=0.0;
   double Sxc=0.0;
   double Sxs=0.0;
   for(int i=0;i<np;i++)
   {
      double cos=MathCos(w*i);
      double sin=MathSin(w*i);
      Sc+=cos;
      Ss+=sin;
      Scc+=cos*cos;
      Sss+=sin*sin;
      Scs+=cos*sin;
      Sx+=(Ext[i+lb]-pv[i]);
      Sxx+=MathPow(Ext[i+lb]-pv[i],2);
      Sxc+=(Ext[i+lb]-pv[i])*cos;
      Sxs+=(Ext[i+lb]-pv[i])*sin;
   }
   Sc/=np;
   Ss/=np;
   Scc/=np;
   Sss/=np;
   Scs/=np;
   Sx/=np;
   Sxx/=np;
   Sxc/=np;
   Sxs/=np;
   if(w==0.0)
   {
      m=Sx;
      c=0.0;
      s=0.0;
   }
   else
   {
      //calculating c, s, and m
      double den=MathPow(Scs-Sc*Ss,2)-(Scc-Sc*Sc)*(Sss-Ss*Ss);
      c=((Sxs-Sx*Ss)*(Scs-Sc*Ss)-(Sxc-Sx*Sc)*(Sss-Ss*Ss))/den;
      s=((Sxc-Sx*Sc)*(Scs-Sc*Ss)-(Sxs-Sx*Ss)*(Scc-Sc*Sc))/den;
      m=Sx-c*Sc-s*Ss;
   }
   return;
}