
#property indicator_separate_window
#property indicator_buffers    4
#property indicator_color1     Red
#property indicator_width1     2
#property indicator_color2     SteelBlue
#property indicator_width2     2
#property indicator_color3     DarkSlateGray
#property indicator_width3     2
#property indicator_color4     Magenta 
#property indicator_style4     STYLE_DASH
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern int    FastMa          = 5;
extern string FastMAModes     = "SMA0,EMA1,SmmMA2,LWMA3";
extern int    FastMaMode      = MODE_EMA;
extern string FastPrices      = "0C,1O,2H,3L,4M,5T,6W";
extern int    FastMaPrice     = PRICE_CLOSE;
extern int    SlowMa          = 21;
extern string SlowMAModes     = "SMA0,EMA1,SmmMA2,LWMA3";
extern int    SlowMaMode      = MODE_EMA;
extern string SlowPrices      = "0C,1O,2H,3L,4M,5T,6W";
extern int    SlowMaPrice     = PRICE_CLOSE;
extern int    SignalMa        = 3;
extern string SignMAModes     = "SMA0,EMA1,SmmMA2,LWMA3";
extern int    SignalMaMode    = MODE_SMA;

extern string _               = "alerts settings";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;
extern string soundFile       = "alert2.wav";

extern string  __              = "arrows settings";
extern bool   ShowArrows       = true;
extern string arrowsIdentifier = "macd arrows";
extern color  arrowsUpColor    = Lime;
extern color  arrowsDnColor    = Red;

//
//
//
//
//

double macdDn[]; 
double macdUp[]; 
double sign[];
double macd[];
double trend[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{

   IndicatorBuffers(5);
   SetIndexBuffer(0,macdDn); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,macdUp); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,macd);
   SetIndexBuffer(3,sign);
   SetIndexBuffer(4,trend);
      
   FastMa   = MathMax(FastMa,1);
   SlowMa   = MathMax(SlowMa,1);
   SignalMa = MathMax(SignalMa,1);
   IndicatorShortName("macd("+FastMa+","+SlowMa+","+SignalMa+")");
return(0);
}

int deinit() 
{ 
   if (ShowArrows) deleteArrows(); 
   
return(0); 
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
  
   for (i=limit; i>=0; i--) macd[i] = iMA(NULL,0,FastMa,0,FastMaMode,FastMaPrice,i) - iMA(NULL,0,SlowMa,0,SlowMaMode,SlowMaPrice,i);
   for (i=limit; i>=0; i--) sign[i] = iMAOnArray(macd,Bars,SignalMa,0,SignalMaMode,i);
   for (i=limit; i>=0; i--) 
   {      
       macdDn[i] = EMPTY_VALUE;
       macdUp[i] = EMPTY_VALUE;
       trend[i]  = trend[i+1];
          if (macd[i] > 0)   trend[i]  =  1;
          if (macd[i] < 0)   trend[i]  = -1;
          if (trend[i] == 1) macdUp[i] = macd[i];
          if (trend[i] ==-1) macdDn[i] = macd[i];
          manageArrow(i);
               
    }
    manageAlerts();
return(0);
} 
      
      //
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Macd ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Macd "),message);
          if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Macd " +" "+message));
          if (alertsSound)   PlaySound(soundFile);
   }
}

//
//
//
//
//

void manageArrow(int i)
{
   if (ShowArrows)
   {
      deleteArrow(Time[i]);
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,241,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,242,true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode, bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i)/2.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
  
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}



