//compiled//
//+------------------------------------------------------------------+
//|                                         MACD_ColorHist_Alert.mq4 |
//|                                    Copyright  2006, Robert Hill |
//|                                                                  |
//Could you give me a price for taking your MACD Color Hist Alert (attached)
// and changing it so that the histogram bars change color when they
// switch from expanding to contracting (and visa versa)
// and also popup an alert that this has happened,
// as well as add an extra alert (popup and email) when the signal line
// goes thru 0 along with a text message on the chart
// that says "the trend has changed to up/down" as the case may be.
//The attached JRSX histograms change color the way I like.
// Perhaps you could add the same alerts as per above to this one as well.
//+------------------------------------------------------------------+
//mod switches, etc. priceless

#property  copyright "Copyright  2006, Robert Hill"
//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 4
#property  indicator_color1  Red
#property  indicator_color2  Green
#property  indicator_color3  LimeGreen
#property  indicator_color4  DarkOliveGreen

#property indicator_style3     2

//---- indicator parameters

extern int FastEMA=12;
extern int SlowEMA=26;
extern int SignalSMA=9;


extern bool SoundON=true;
extern bool EmailON=false;

extern bool ocma0cross  =true;
extern bool ocmaDirect  =false;

extern bool macdAlert      =true;
extern bool useMacdSignLn  =false;
extern int  alertsOnBar    =1;

extern bool   CommentON    =true;
extern double OsmaMultipl  =2.5;

//---- indicator buffers
double     MACD_buffer[];
double     Signal_buffer[];
double HistogramBufferUp[];
double HistogramBufferDown[];


bool HistAboveZero   = false;
bool HistBelowZero   = false;
bool HistDirectDn    = false;
bool HistDirectUp    = false;
bool MACDAboveZero   = false;
bool MACDBelowZero   = false;

//---- variables

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- drawing settings
//   IndicatorBuffers(3);
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexBuffer(3,MACD_buffer);
   SetIndexDrawBegin(3,SlowEMA);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,Signal_buffer);
   SetIndexDrawBegin(2,SignalSMA);
   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(1,HistogramBufferUp);
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(0,HistogramBufferDown);
//   SetIndexDrawBegin(2,SlowEMA + SignalSMA);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD ("+FastEMA+","+SlowEMA+","+SignalSMA+") ");
   SetIndexLabel(3,"MACD");
   SetIndexLabel(2,"Signal");
   SetIndexLabel(1,"HistogramUp");
   SetIndexLabel(0,"HistogramDn");
   
//---- initialization done
   return(0);
  }
  
 
int deinit()
{
   Comment ("");
   return(0);
}  
  
  
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
   int limit;
   double current, prev;

   int counted_bars=IndicatorCounted();
//---- check for possible errors
   if(counted_bars<0) return(-1);
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- macd counted in the 1-st buffer
   for(int i=0; i<limit; i++)
      MACD_buffer[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
//---- signal line counted in the 2-nd buffer
   for(i=0; i<limit; i++)
      Signal_buffer[i]=iMAOnArray(MACD_buffer,Bars,SignalSMA,0,MODE_SMA,i);

   for(i=0; i<limit; i++)
   {
      HistogramBufferUp[i] = 0;
      HistogramBufferDown[i] = 0;
      current = (MACD_buffer[i] - Signal_buffer[i]) *OsmaMultipl;
      prev = (MACD_buffer[i+1] - Signal_buffer[i+1])*OsmaMultipl;
      if (current > prev)
      {
        HistogramBufferUp[i] = current;
        HistogramBufferDown[i] = 0.0;
     
      }
      else
      {
        HistogramBufferDown[i] = current;
        HistogramBufferUp[i] = 0.0;
      
      }
      
      if (CommentON && MACDAboveZero) Comment ("\nThe trend has changed to UP");
      if (CommentON && MACDBelowZero) Comment ("\nThe trend has changed to DOWN");
   
      
      
 if (i == alertsOnBar)
   {
// Check for Histogram crossing 0 line
        if (HistogramBufferUp[i]>0 && HistogramBufferUp[i+1]<0)
        {
// Cross up
         if (HistAboveZero == false)
         {
           HistAboveZero=true;
           HistBelowZero=false;
           if (SoundON&&ocma0cross) Alert("MACDosma Histogram Crossed0 up","\n Time=",TimeToStr(CurTime(),TIME_DATE)," ",TimeHour(CurTime()),":",TimeMinute(CurTime()),"\n Symbol ",Symbol()," Period M",Period());
           if (EmailON&&ocma0cross) SendMail("MACDosma Histogram Crossed0 up", "MACD Crossed up, Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol "+Symbol()+" Period M"+Period());
         }
        }
        else if (HistogramBufferDown[i]<0 && HistogramBufferDown[i+1]>0)
        {
// Cross down
         if (HistBelowZero == false)
         {
          HistBelowZero=true;
          HistAboveZero=false;
          if (SoundON&&ocma0cross) Alert("MACDosma Histogram Crossed0 down","\n Date=",TimeToStr(CurTime(),TIME_DATE)," ",TimeHour(CurTime()),":",TimeMinute(CurTime()),"\n Symbol=",Symbol()," Period M",Period());
          if (EmailON&&ocma0cross) SendMail("MACDosma Histogram Crossed0 down","MACD Crossed Down, Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period M"+Period());
         }
        }
        else if (HistogramBufferDown[i]<0 && HistogramBufferDown[i+1]>0)
        {
// Cross down
         if (HistBelowZero == false)
          {
          HistBelowZero=true;
          HistAboveZero=false;
          if (SoundON&&ocma0cross) Alert("MACDosma Histogram Crossed0 down","\n Date=",TimeToStr(CurTime(),TIME_DATE)," ",TimeHour(CurTime()),":",TimeMinute(CurTime()),"\n Symbol=",Symbol()," Period M",Period());
          if (EmailON&&ocma0cross) SendMail("MACDosma Histogram Crossed0 down","MACD Crossed Down, Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period M"+Period());
          }
         }



        else if (HistogramBufferUp[i]>HistogramBufferDown[i+1])
           {
           if (SoundON&&ocmaDirect&&!HistDirectUp) 
               Alert("MACDosma Histogram changed direction: UP","\n Time=",TimeToStr(CurTime(),TIME_DATE)," ",TimeHour(CurTime()),":",TimeMinute(CurTime()),"\n Symbol=",Symbol()," Period M",Period());
           if (EmailON&&ocmaDirect&&!HistDirectUp) 
               SendMail("MACDosma Histogram changed direction: UP", "MACD histo dir. up, Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period M"+Period());
           HistDirectUp= true; HistDirectDn= false; 
           }
          

        else if (HistogramBufferDown[i]<HistogramBufferUp[i+1])
           {
           if (SoundON&&ocmaDirect&&!HistDirectDn) 
               Alert("MACDosma Histogram changed direction: Down","\n Time=",TimeToStr(CurTime(),TIME_DATE)," ",TimeHour(CurTime()),":",TimeMinute(CurTime()),"\n Symbol=",Symbol()," Period M",Period());
           if (EmailON&&ocmaDirect&&!HistDirectDn) 
               SendMail("MACDosma Histogram changed direction: Down", "MACD histo dir. up, Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period M"+Period());
           HistDirectDn= true; HistDirectUp= false;
           }
        
        
        
        
        
// Check for MACD Signal line crossing 0 line

         double macd0,macd1; 
      
      if(macdAlert &&!useMacdSignLn) { macd0= MACD_buffer[i];   macd1=MACD_buffer[i+1];}
      if(macdAlert && useMacdSignLn) { macd0= Signal_buffer[i]; macd1=Signal_buffer[i+1];}
        
      if (macd0 > 0 && macd1 < 0)
      {
// Cross up
         if (MACDAboveZero == false)
         {
           MACDAboveZero=true;
           MACDBelowZero=false;
           if (SoundON) Alert("MACD  Crossed 0 up","\n Time=",TimeToStr(CurTime(),TIME_DATE)," ",TimeHour(CurTime()),":",TimeMinute(CurTime()),"\n Symbol=",Symbol()," Period M",Period());
           if (EmailON) SendMail("MACD Crossed 0 up", "MACD Crossed up, Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period M"+Period());
         }
        }
        else if (macd0 < 0 && macd1 > 0)
        {
// Cross down
         if (MACDBelowZero == false)
         {
          MACDBelowZero=true;
          MACDAboveZero=false;
          if (SoundON) Alert("MACD Crossed 0 down","\n Date=",TimeToStr(CurTime(),TIME_DATE)," ",TimeHour(CurTime()),":",TimeMinute(CurTime()),"\n Symbol=",Symbol()," Period M",Period());
          if (EmailON) SendMail("MACD Crossed 0 down","MACD Crossed Down, Date="+TimeToStr(CurTime(),TIME_DATE)+" "+TimeHour(CurTime())+":"+TimeMinute(CurTime())+" Symbol="+Symbol()+" Period M"+Period());
          }
         }

     }

      
   }
      
//---- done
   return(0);
  }