//------------------------------------------------------------------
#property copyright "mladen"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  clrLimeGreen
#property indicator_color2  clrOrange
#property indicator_color3  clrOrange
#property indicator_width1  3
#property indicator_width2  3
#property indicator_width3  3
#property strict

//
//
//
//
//

extern int                mkperiod   = 14;          // Kahler smoothing period
extern int                mfperiod   = 12;          // Macd fast period
extern int                msperiod   = 26;          // Macd slow period
extern int                mtperiod   =  9;          // Macd signal period
extern ENUM_MA_METHOD     mtmethod   = MODE_EMA;    // Macd signal smoothing method
extern ENUM_APPLIED_PRICE price      = PRICE_CLOSE; // Price
extern int                linesWidth =  3;          // Lines width
extern double             fastr      =  8;          // Fast ratio
extern double             slowr      = 22;          // Slow ratio
extern bool               alertsOn        = false;  // Turn alerts on
extern bool               alertsOnCurrent = true;   // Alerts on currnt (still opened) bar
extern bool               alertsMessage   = true;   // Alerts should show pop-up message
extern bool               alertsSound     = false;  // Alerts should play alert sound
extern bool               alertsPushNotif = false;  // Alerts should send push notification
extern bool               alertsEmail     = false;  // Alerts should send email
extern bool               showArrows       = false; // Display arrows on chart?
extern string             arrowsIdentifier = "MacdDKArrows1";  // Unique identifier for arrows
extern double             arrowsUpperGap   = 0.5;              // Upper arrow gap
extern double             arrowsLowerGap   = 0.5;              // Lower arrow gap
extern color              arrowsUpColor    = clrLimeGreen;     // Upper arrow color
extern color              arrowsDnColor    = clrOrange;        // Lower arrow color
extern int                arrowsUpCode     = 241;              // Upper arrow code
extern int                arrowsDnCode     = 242;              // Lower arrow code

double buffer[];
double bufferda[];
double bufferdb[];
double trend[];
double macd[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,buffer,  INDICATOR_DATA); SetIndexStyle(0,EMPTY,EMPTY,linesWidth);
   SetIndexBuffer(1,bufferda,INDICATOR_DATA); SetIndexStyle(1,EMPTY,EMPTY,linesWidth);
   SetIndexBuffer(2,bufferdb,INDICATOR_DATA); SetIndexStyle(2,EMPTY,EMPTY,linesWidth);
   SetIndexBuffer(3,trend   ,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,macd    ,INDICATOR_CALCULATIONS);
   IndicatorShortName("macd digital Kahler ("+(string)mkperiod+","+(string)mfperiod+","+(string)msperiod+(string)mtperiod+","+")");
   return(0);
}
int deinit() 
{  
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0); 
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);

   //
   //
   //
   //
   //

   if (trend[limit]==-1) CleanPoint(limit,bufferda,bufferdb);
   for(int i=limit; i>=0; i--) macd[i] = iMA(NULL,0,mfperiod,0,MODE_EMA,price,i)-iMA(NULL,0,msperiod,0,MODE_EMA,price,i);
   for(int i=limit; i>=0; i--)
   {
      double fast_k = macd[i];
      double slow_k = iMAOnArray(macd,0,mtperiod,0,mtmethod,i);
      double temp   = 0;
         if ((slowr*slow_k+fastr*fast_k)/(fastr+slowr)>0.0) temp= 1;
         if ((slowr*slow_k+fastr*fast_k)/(fastr+slowr)<0.0) temp=-1;
            buffer[i]   = iEma(temp,mkperiod,i);
            bufferda[i] = EMPTY_VALUE;
            bufferdb[i] = EMPTY_VALUE;
            trend[i]    = trend[i+1];
      
         //
         //
         //
         //
         //
         
         if (buffer[i]>buffer[i+1]) trend[i] =  1;
         if (buffer[i]<buffer[i+1]) trend[i] = -1;
         if (trend[i] == -1) PlotPoint(i,bufferda,bufferdb,buffer);
         if (showArrows)
         {
            ObjectDelete(arrowsIdentifier+":"+(string)Time[i]);
            if (trend[i] != trend[i+1])
            {
               if (trend[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
               if (trend[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
            }
         }
   }      
   manageAlerts();
   return(0);
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" macd digital Kahler slope changed to "+doWhat;
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(Symbol()+" macd digital Kahler",message);
          if (alertsPushNotif) SendNotification(message);
          if (alertsSound)     PlaySound("alert2.wav");
   }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workEma[][1];
double iEma(double tprice, double eperiod, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+eperiod);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(tprice-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
         {
            if (first[i+2] == EMPTY_VALUE) 
                  { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
            else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
         }
   else  { first[i] = from[i]; second[i] = EMPTY_VALUE; }
}