//+------------------------------------------------------------------+
//+                       MACD_DJL.mq4                               +
//+              MACD plus 4 colour Histogram                        +
//+     By David Lewis, Southern Cross Forex Limited, 2008           +
//+------------------------------------------------------------------+

#property copyright "Southern Cross Forex Limited, copyright 2008"
#property link "david.lewis@southerncrossforex.com"

//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 6
#property  indicator_color1  Lime
#property  indicator_color2  Green
#property  indicator_color3  Red
#property  indicator_color4  Maroon
#property  indicator_color5  Black
#property  indicator_color6  Aqua
#property  indicator_width1  3
#property  indicator_width2  3
#property  indicator_width3  3
#property  indicator_width4  3
#property  indicator_width5  1
#property  indicator_width6  1

//---- indicator parameters
extern int FastEMA=12;
extern int SlowEMA=26;
extern int SignalEMA=9;
extern int HistoMultiplier=3;

//---- indicator buffers

double     ind_buffer1[], ind_buffer1s[];
double     ind_buffer2[], ind_buffer2s[];
double     MacdBuffer[];
double     SignalBuffer[];
double     ind_buffer3[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- 1 additional buffer used for counting.
   IndicatorBuffers(7);  
 
//---- drawing settings - Histogram
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,3);
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID,3);
   SetIndexStyle(2,DRAW_HISTOGRAM,STYLE_SOLID,3);
   SetIndexStyle(3,DRAW_HISTOGRAM,STYLE_SOLID,3);
   
//---- 3 indicator buffers mapping
   if(!SetIndexBuffer(0,ind_buffer1) &&
      !SetIndexBuffer(1,ind_buffer1s) &&
      !SetIndexBuffer(2,ind_buffer2) &&
      !SetIndexBuffer(3,ind_buffer2s) &&
      !SetIndexBuffer(6,ind_buffer3))
      Print("cannot set indicator buffers!");
 //---- drawing settings - MACD and Signal EMAs
   SetIndexStyle(4,DRAW_LINE);
   SetIndexStyle(5,DRAW_LINE);
   SetIndexDrawBegin(1,SignalEMA);
   IndicatorDigits(Digits+1);
//---- indicator buffers mapping
   SetIndexBuffer(4,MacdBuffer);
   SetIndexBuffer(5,SignalBuffer);  
 //---- name for DataWindow and indicator subwindow label
   IndicatorShortName("MACD_DJL1("+FastEMA+","+SlowEMA+","+SignalEMA+")");
   SetIndexLabel(4,"MACD");
   SetIndexLabel(5,"Signal");        
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
  {
   int limit;
   int counted_bars=IndicatorCounted();
   double prev,current;
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- macd counted in the 1-st buffer
   for(int i=0; i<limit; i++)
      MacdBuffer[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
//---- signal line counted in the 2-nd buffer
   for(i=0; i<limit; i++)
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalEMA,0,MODE_EMA,i);
      }
//---- MACD EMAs done
   {
//---- check for possible errors
   if(counted_bars<0) return(-1);
   //---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;
//---- macd counted in the 1-st additional buffer
   for(i=0; i<limit; i++)
      ind_buffer3[i]=MacdBuffer[i]-SignalBuffer[i];
//---- dispatch values between 2 buffers
   bool up=true;
   for(i=limit-1; i>=0; i--)
     {
      current=ind_buffer3[i];
      prev=ind_buffer3[i+1];
      if (((current<0)&&(prev>0))||(current<0))   up= false;    
      if (((current>0)&&(prev<0))||(current>0))   up= true;      
      
      if(!up)
        {
         if(current > prev)
           {
            ind_buffer2s[i]=current*HistoMultiplier;
            ind_buffer2[i]=0.0;
            ind_buffer1[i]=0.0;
            ind_buffer1s[i]=0.0;             
           }        
         else
           {
            ind_buffer2[i]=current*HistoMultiplier;
            ind_buffer2s[i]=0.0;
            ind_buffer1[i]=0.0;
            ind_buffer1s[i]=0.0;            
           }             
        }
        else
        {
         if(current < prev)
           {
            ind_buffer1s[i]=current*HistoMultiplier;
            ind_buffer1[i]=0.0;
            ind_buffer2[i]=0.0;
            ind_buffer2s[i]=0.0;
           }
        else
           {
            ind_buffer1[i]=current*HistoMultiplier;
            ind_buffer1s[i]=0.0;
            ind_buffer2[i]=0.0;
            ind_buffer2s[i]=0.0;
            }
            }           
         }       
     }
//---- Histogram done
   return(0);
  }
//+------------------------------------------------------------------+