//+------------------------------------------------------------------+
//|                                                     MACD mtf.mq4 |
//|                        Copyright 2004, MetaQuotes Software Corp. |
//+------------------------------------------------------------------+
#property  copyright "Copyright  2004, MetaQuotes Software Corp."
#property  link      "http://www.metaquotes.net/"

#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  clrBlack
#property  indicator_color2  clrRed
#property  indicator_width1  3
#property  indicator_width2  2

extern ENUM_TIMEFRAMES    TimeFrame            = PERIOD_CURRENT;
extern double             FastEMA              = 12;
extern double             SlowEMA              = 26;
extern double             SignalSMA            = 9;

extern bool               alertsOn             = false;
extern bool               alertsOnCurrent      = true;
extern bool               alertsMessage        = true;
extern bool               alertsSound          = false;
extern bool               alertsEmail          = false;

extern bool               ShowLines            = true;
extern bool               linesOnFirst         = false;
extern string             verticalLinesID      = "MACD mtf Lines";
extern color              verticalLinesUpColor = Blue;
extern color              verticalLinesDnColor = Red;
extern ENUM_LINE_STYLE    verticalLinesStyle   = STYLE_SOLID;
extern int                verticalLinesWidth   = 2;
extern bool               Interpolate          = true;
extern int                LookingBarsLines     =3000;

double MacdBuffer[],SignalBuffer[];
string indicatorFileName;
bool   returnBars;
int    maxArrows;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(2);   
   SetIndexBuffer(0,MacdBuffer);    SetIndexLabel(0,"MACD");
   SetIndexBuffer(1,SignalBuffer);  SetIndexLabel(1,"Signal");
     
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
   
   IndicatorShortName(timeFrameToString(TimeFrame)+" MACD mtf ("+FastEMA+","+SlowEMA+","+SignalSMA+")");
   
   return(0);
  }

//----------------------------

int deinit()
{ if (ShowLines) deleteLines();
return(0);
}

//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+

int start()
  {
   int counted_bars=IndicatorCounted();
   int limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit=MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { MacdBuffer[0] = MathMin(limit+1,Bars-1); return(0); }

      
   if (TimeFrame == Period())
   {
      for (int i=limit;i>=0;i--)

         {
            MacdBuffer[i]   = iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
            SignalBuffer[i] = iMAOnArray(MacdBuffer,Bars,SignalSMA,0,MODE_SMA,i);
         }
  
  
      if (ShowLines)
                 {
                   deleteLines();
                   for (i=0; i<LookingBarsLines ;i++)
                     {
                        if (MacdBuffer[i]>SignalBuffer[i] && MacdBuffer[i+1]<SignalBuffer[i+1]) drawLine(i,verticalLinesUpColor);
                        if (MacdBuffer[i]<SignalBuffer[i] && MacdBuffer[i+1]>SignalBuffer[i+1]) drawLine(i,verticalLinesDnColor);
                     }
                 }      
                  
   manageAlerts();
   return(0);
   
  }

//-----------------------------------
 
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         MacdBuffer[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,FastEMA,SlowEMA,SignalSMA,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,ShowLines,linesOnFirst,verticalLinesID,verticalLinesUpColor,verticalLinesDnColor,verticalLinesStyle,verticalLinesWidth,0,y);
         SignalBuffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,FastEMA,SlowEMA,SignalSMA,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,ShowLines,linesOnFirst,verticalLinesID,verticalLinesUpColor,verticalLinesDnColor,verticalLinesStyle,verticalLinesWidth,1,y);
               
         if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

            datetime time = iTime(NULL,TimeFrame,y);
               for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(int k = 1; k < n; k++) 
                     {
                        MacdBuffer[i+k] = MacdBuffer[i] + (MacdBuffer[i+n] - MacdBuffer[i])*k/n;
                        SignalBuffer[i+k] = SignalBuffer[i] + (SignalBuffer[i+n] - SignalBuffer[i])*k/n;
                     }
   }
return(0);
}

//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
//-------------------------------------------------------------------

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
          if (MacdBuffer[whichBar]>SignalBuffer[whichBar] && MacdBuffer[whichBar+1]<SignalBuffer[whichBar+1])doAlert("up");
          if (MacdBuffer[whichBar]<SignalBuffer[whichBar] && MacdBuffer[whichBar+1]>SignalBuffer[whichBar+1])doAlert("down");
   }
}

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       message =  StringConcatenate(timeFrameToString(Period())+" "+Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," MACD crossed ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" MACD mtf",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//---------------------------------------------------------------------

void drawLine(int i,color theColor)
{
      maxArrows++;
      string name = StringConcatenate(verticalLinesID,maxArrows);
         
      int add = 0; if (!linesOnFirst) add = _Period*60-1;
      ObjectCreate(name,OBJ_VLINE,0,Time[i]+add,0);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_STYLE,verticalLinesStyle);
         ObjectSet(name,OBJPROP_WIDTH,verticalLinesWidth);
         ObjectSet(name,OBJPROP_BACK,true);
}

void deleteLines()
{
   while(maxArrows>0) { ObjectDelete(StringConcatenate(verticalLinesID,maxArrows)); maxArrows--; }
}
